<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * TONE2tTONE search class
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    Contenido Backend classes
 * @version    1.0.0
 * @author     Viktor Lehmann
 * @copyright  TONE2TONE <www.tone2tone.com>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.tone2tone.com
 * @since      file available since contenido release <= 4.9.12
 *
 * {@internal
 *   created  2018-12-08 Viktor Lehmann
 *   modified 2018-12-08 Viktor Lehmann
 *   modified 2018-12-08 Viktor Lehmann

 *
 *   $Id: class.fullsearch.php 2018-12-08 Viktor Lehmann $:
 * }}
 *
 */

defined('CON_FRAMEWORK') || die('Illegal call: Missing framework initialization - request aborted.');

/**
 * Fullsearch management class
 */
class FullsearchCollection extends ItemCollection
{

    /**
     * Constructor Function
     * @param none
     */

    /**
     * Prefix to be used for Fullsearch <uniqueKey> in order to distinguish documents
     * from different sources.
     *
     * @var string
     */
    const ID_PREFIX = 'contenido_article_';

    /**
     * IDs of articles to be updated / added / deleted.
     *
     * @var array
     */
    private $_articleIds = array();
    private $_feuUserName = "";
    private $_feuUserPassword = "";
    private $_tagging = false;
    private $_idartLogin = false;
    private $_clients = array();
    private $_searchtable = "";

    public function __construct(array $articleIds = array())
    {

        global $cfg, $cfgClient;
        //print_r($cfg);

        /*
        Note to myself:
        der Konstruktor braucht hier ein Array, damit später der Einzelartikel nach dem Abspeichern durch den Redakteur
        abgespeichert werden kann. Für die Gesamtindizierung ist er aber Käse. Daher muss beim reindex-Aufruf dennoch ein
        leeres Array übergeben werden, damit der Konstruktor nicht abstürzt.
         */

        parent::__construct($cfg["tab"]["fullsearch"], "idfullsearch");
        $this->_setItemClass("Fullsearch");

        $this->_articleIds = $articleIds;

        //print "there";

        //find all active clients and dump them into $this for future reference. Array keys hold idclient.
        //then add client specific login-idart to array, needed for curl later
        $allClients = new cApiClientCollection();
        $this->_clients = $allClients->getAccessibleClients();

        foreach ($this->_clients as $key => $value) {
            $huhu = new cApiClient($key);
            //print "key" . $key . "effective" . $huhu->getProperty('login', 'idart', $key);
            $this->_clients[$key]["loginidart"] = $huhu->getProperty('login', 'idart', $key);
        }

        //var_dump($this->_clients);

        // get global variables from registry (the nice way)
        $this->_cfg = cRegistry::getConfig();
        $this->_db = cRegistry::getDb();
        //$this->_client = cRegistry::getClientId();
        $this->_lang = cRegistry::getLanguageId();
        $this->_idcat = cRegistry::getCategoryId();
        $this->_idart = cRegistry::getArticleId();
        $this->_sess = cRegistry::getSession();
        $this->_searchtable = $cfg["tab"]["fullsearch"];

        //$this->_frontendpath = $cfgClient[1]["htmlpath"]["frontend"];

        // Adjust this values to your installation if necessary
        $this->_feuUserName = "fullsearchuser";
        $this->_feuUserPassword = "*FuLLseaRch*User*";
        $this->_tagging = ($cfg["tab"]["pica_alloc"] != '') ? true : false;
        $this->_idartLogin = getEffectiveSetting('login', 'idart', ''); // is empty string in case of not set

    }

    private function _isIndexable($idartlang)
    {

        // What about time managment?
        $articleLanguage = new cApiArticleLanguage($idartlang);
        if (!$articleLanguage->isLoaded()) {
            return false;
        } else if (1 != $articleLanguage->get('online')) {
            return false;
        } else if (1 != $articleLanguage->get('searchable')) {
            return false;
        } else {
            return true;
        }
    }

    /*
     * Creates a new booking
     */
    public function create()
    {
        global $client, $auth;

        $item = parent::createNewItem();
        //$item->set("idfullsearch", 6);
        //$item->set("", 800);

        $item->store();

        return $item;
    }

    public function reindex()
    {
        $cfg = cRegistry::getConfig();

        $idclient = cRegistry::getClientId();
        $idclient = cSecurity::toInteger($idclient);

        $idlang = cRegistry::getLanguageId();
        $idlang = cSecurity::toInteger($idlang);

        // statement is not correct if articles are related to more than one category.
        $db = cRegistry::getDb();
        $db->query("SELECT
                art.idclient
                , art_lang.idlang
                , cat_art.idcat
                , cat_lang.idcatlang
                , art_lang.idart
                , art_lang.idartlang
            FROM
                `" . $cfg['tab']['art_lang'] . "` AS art_lang
            INNER JOIN
                `" . $cfg['tab']['art'] . "` AS art
            ON
                art_lang.idart = art.idart
            INNER JOIN
                `" . $cfg['tab']['cat_art'] . "` AS cat_art
            ON
                art_lang.idart = cat_art.idart
            INNER JOIN
                `" . $cfg['tab']['cat_lang'] . "` AS cat_lang
            ON
                cat_art.idcat = cat_lang.idcat
                AND art_lang.idlang = cat_lang.idlang

            ORDER BY
                art_lang.idartlang
            ");

        // original sql statement had where-clause:
        // WHERE art.idclient = $idclient AND art_lang.idlang = $idlang

        $this->_articleIds = array();
        while ($db->nextRecord()) {
            array_push($this->_articleIds, array(
                'idclient' => $db->f('idclient'),
                'idlang' => $db->f('idlang'),
                'idcat' => $db->f('idcat'),
                'idcatlang' => $db->f('idcatlang'),
                'idart' => $db->f('idart'),
                'idartlang' => $db->f('idartlang'),
            ));
        }

        /*
        print "<pre>";
        print_r($articleIds);
        print "</pre>";
         */
        $this->updateArticles();

        //$cGuiNotification = new cGuiNotification();
        //return $cGuiNotification->returnNotification(cGuiNotification::LEVEL_OK, 'core was reindexed');
    }

    private function _getContent($idartlang)
    {

        // 'CMS_IMG', 'CMS_LINK', 'CMS_LINKTARGET', 'CMS_SWF'
        $cms = "'CMS_RAW','CMS_HTMLHEAD','CMS_HTML','CMS_TEXT','CMS_IMGDESCR',"
            . "'CMS_LINKDESCR','CMS_HEAD','CMS_LINKTITLE','CMS_LINKEDIT',"
            . "'CMS_RAWLINK','CMS_IMGEDIT','CMS_IMGTITLE','CMS_SIMPLELINKEDIT',"
            . "'CMS_HTMLTEXT','CMS_EASYIMGEDIT','CMS_DATE','CMS_TEASER',"
            . "'CMS_FILELIST','CMS_IMGEDITOR','CMS_LINKEDITOR','CMS_PIFAFORM'";

        // exclude certain content types from indexing
        // like in conMakeArticleIndex & conGenerateKeywords
        $db = cRegistry::getDb();
        $db->query("
            SELECT
                con_type.type
                , con_content.typeid
                , con_content.value
            FROM
                con_content
            INNER JOIN
                con_type
            ON
                con_content.idtype = con_type.idtype
            WHERE
                con_content.idartlang = $idartlang
                AND con_type.type IN ($cms)
            ORDER BY
                con_content.idtype
                , con_content.typeid
            ;");

        $content = array();
        while (false !== $db->nextRecord()) {
            $value = $db->f('value');
            //$value = utf8_encode($value);
            $value = strip_tags($value);
            //$value = html_entity_decode($value);
            $value = html_entity_decode($value, ENT_QUOTES, 'UTF-8');
            $value = trim($value);

            $content[$db->f('type')][$db->f('typeid')] = $value;
        }

        // TODO check first alternative:
        // cInclude('includes', 'functions.con.php');
        // $content = conGetContentFromArticle($this->_idartlang);
        // TODO check second alternative:
        // $articleLanguage = new cApiArticleLanguage($this->_idartlang);
        // if (!$articleLanguage->isLoaded()) {
        // throw new cException('article could not be loaded');
        // }
        // $content = $articleLanguage->getContent();

        return $content;
    }

    /**
     * Delete all CONTENIDO article documents that are aggregated as
     * $this->_articleIds.
     *
     * @throws SolrClientException if Solr delete request failed
     */
    public function deleteArticles()
    {
        //print " -- deletion: start -- ";
        //print "<pre>"; print_r($this->_articleIds); print "</pre>";
        foreach ($this->_articleIds as $articleIds) {
            //$oFullsearchCollection = new FullsearchCollection();
            //$result = $oFullsearchCollection->getIdsByWhereClause("id_art_lang = ".(int)$articleIds['idartlang']);
            $result = self::getIdsByWhereClause("id_art_lang = " . (int) $articleIds['idartlang']);
            //print_r($result);
            if (!empty($result)) {
                self::delete($result[0]);
                //print "<br>deleted idartlang: ".$articleIds['idartlang'];
            } else {
                //print " -- nothing to delete -- ";
            }
        }
        //print " -- deletion: done -- ";
    }

    public function deleteIndex()
    {
        //print " -- truncate: start -- ";
        $db2 = cRegistry::getDb();

        $sql = "TRUNCATE TABLE " . $this->_searchtable;
        $db2->query($sql);
        //print " -- truncate: done -- ";
    }

    public function updateArticles()
    {

        // Always delete articles from index, even if article should not be
        // indexed it might have been indexed before
        // What happens if an article could not be deleted cause it was not
        // indexed before? does this throw an exception? if yes an article
        // could never been indexed!
        try {
            $this->deleteArticles();
        } catch (cException $e) {
            // ignore exception so that articles can be indexed nonetheless
        }

        // add articles to index
        // will be skipped if article is not indexable
        $this->addArticles();
    }

    public function updateSingleArticle(array $articleUpdate)
    {

        // Always delete articles from index, even if article should not be
        // indexed it might have been indexed before
        // What happens if an article could not be deleted cause it was not
        // indexed before? does this throw an exception? if yes an article
        // could never been indexed!
        if (!empty($articleUpdate)) {
            $this->_articleIds = $articleUpdate;
            try {
                $this->deleteArticles();
            } catch (cException $e) {
                // ignore exception so that articles can be indexed nonetheless
            }
            $this->addArticles();
        }

        // add articles to index
        // will be skipped if article is not indexable

    }

    /**
     * If the current articles are indexable for each article a new index
     * document will be created and filled with its content and eventually
     * be added to the index.
     *
     * @throws cException if Solr add request failed
     */
    public function addArticles()
    {

        //print " -- addArticles: start -- ";

        //$lmsSite = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['SERVER_NAME'];
        // $lmsSite = $cfgClient[1]["htmlpath"]["frontend"];

        //$toAdd = array();
        //print "innn";
        foreach ($this->_articleIds as $articleIds) {

            // skip if article should not be indexed
            if (!$this->_isIndexable($articleIds['idartlang'])) {
                //continue;
            }

            //print "<pre>"; print_r($articleIds); print "</pre>";

            /*
            if (!isset($toAdd[$articleIds['idlang']])) {
            $toAdd[$articleIds['idlang']] = array(
            'idclient' => $articleIds['idclient'],
            'documents' => array()
            );
            }
             */

            // is Article an IncludeArticle? if idcat = 129: yes, else no
            /* unwichtig
            $categoryHelper = cCategoryHelper::getInstance();
            $categoryHelper->setAuth(cRegistry::getAuth());
            $idcatlevel = $categoryHelper->getCategoryLevel($articleIds['idcat']);
            $idcatparents = $categoryHelper->getParentCategoryIds($articleIds['idcat']); // 20 steht für die max. Anzahl Level, die ermittelt werden
            $articleInclude = in_array( "129", $idcatparents) ? 1 : 0 ;
             */

            // get article content to be indexed
            $articleContent = $this->_getContent($articleIds['idartlang']);

            // create input document
            //$solrInputDocument = new SolrInputDocument();
            //$oFullsearchCollection = new FullsearchCollection();
            //$handleEntry = $oFullsearchCollection->createNewItem($articleIds['idartlang']);
            $handleEntry = self::createNewItem($articleIds['idartlang']);

            //$handleEntry->set('idfullsearch', $articleIds['idartlang']);
            //$handleEntry->store();
            //print_r($handleEntry);

            //$handleEntry->set('idfullsearch', $articleIds['idartlang']);
            //$handleEntry->store();
            $handleEntry->set('source', 'contenido_article');
            $handleEntry->set('url', cUri::getInstance()->build(array(
                'idart' => $articleIds['idart'],
                'lang' => $articleIds['idlang'],
            )));
            //$handleEntry->set('id', self::ID_PREFIX . $articleIds['idartlang']);
            // $handleEntry->set('raise_exception', 'uncomment this
            // to raise an exception');
            // add IDs
            $handleEntry->set('id_client', $articleIds['idclient']);
            $handleEntry->set('id_lang', $articleIds['idlang']);
            $handleEntry->set('id_cat', $articleIds['idcat']);
            $handleEntry->set('id_art', $articleIds['idart']);
            $handleEntry->set('id_cat_lang', $articleIds['idcatlang']);
            $handleEntry->set('id_art_lang', $articleIds['idartlang']);

            $catSettings = new cApiCategoryLanguage();
            $catSettings->loadByPrimaryKey($articleIds['idcatlang']);
            $artSettings = new cApiArticleLanguage();
            $artSettings->loadByPrimaryKey($articleIds['idartlang']);

            $handleEntry->set('cat_public', $catSettings->get("public")); // Kategorie: geschlossener Bereich ja = 0, nein = 1
            $handleEntry->set('cat_visible', $catSettings->get("visible")); // Kategorie: sichtbar ja = 0, nein = 1 (sagt aber nix über Sichtbarkeit des Artikels)
            $handleEntry->set('art_online', $artSettings->get("online")); // Artikel: sichtbar ja = 1, nein = 0
            $handleEntry->set('art_searchable', $artSettings->get("searchable")); // Artikel: suchbar ja = 1, nein = 0
            $handleEntry->set('sortID', $artSettings->get("artsort"));
            $handleEntry->set('articleInclude', $articleInclude); // is Article an IncludeArticle? if idcat = 129: 1, else 0

            // find all possible FEU groups first
            $feugroups = new cApiFrontendGroupCollection();
            $feugroupsArray = $feugroups->getAllIds();

            // now check category permissions against FEU-Groups (first param in checkPerm). Need to store them
            // has to be checked against "cat_public", or all open categories will be marked with a FEUgroup-ID when a FEU-Group is set to "Kategorie-Zugriff (alle)"
            $feupermission = new cApiFrontendPermissionCollection();
            $accessGroups = array();
            foreach ($feugroupsArray as $key => $value) {
                if ($feupermission->checkPerm($value, 'category', 'access', $articleIds['idcatlang']) && $catSettings->get("public") == 0) {
                    array_push($accessGroups, $value);
                }
            }
            $handleEntry->set('accessGroups', "@@@".implode("@@@", $accessGroups)."@@@");
            /*
            for ($i = 1; $i <= 7; $i++) {
            if ( $feupermission->checkPerm( $i, 'category', 'access', $articleIds['idcatlang'] )  ) $handleEntry->set('accessGroups', $i);
            }
             */
/*
print "<pre>";
print_r($articleIds);
print "</pre>";
 */

            // add content one by one
            foreach ($articleContent as $type => $typeContent) {

                // field names in Solr should always be lowercase!
                $type = strtolower($type);

                // == sort content of a certain content type by their typeids
                // This is important so that the most prominent headline can be
                // displayed first.
                ksort($typeContent);

                // add each content entry seperatly (content type fields are
                // defined as multiValued)
                $contentSeparator = array();
                foreach ($typeContent as $typeid => $contentEntry) {
                    $contentEntry = trim($contentEntry);
                    if (0 < strlen($contentEntry)) {
                        //$handleEntry->set($type, $contentEntry);
                        $contentSeparator[$type][] = $contentEntry;
                    }
                }
                foreach ($contentSeparator as $type => $contentEntry) {
                    $handleEntry->set($type, implode("@@@", $contentEntry));
                }

            }

            /* do not use Article Include for Cuckoo right now

            $artIncContainers = array();
            $artIncIdartlangs = array();

            if (isset($articleContent['CMS_RAW'])) {
            foreach ($articleContent['CMS_RAW'] as $typeid => $contentid) {
            $thisraw = unserialize($contentid);
            $thisrawcheck = serialize($thisraw);

            if ( $thisrawcheck != $contentid) {
            $thisraw = (array)json_decode($contentid);
            foreach ( $thisraw as $key => $value ) {

            $thisvalue = (array)$value;
            if ($thisvalue["moduleId"] == "58") $artIncContainers[] = $thisvalue["state"];

            }

            } elseif ( strpos($contentid, "a:2:") !== FALSE ) {
            $getidart = new cApiCategoryArticle();
            $getidart-> loadByPrimaryKey($thisraw[2]);
            $thisidart = $getidart->get("idart");
            //print "idart:". $thisidart ."  xx  ";
            $newidartlang = new cApiArticleLanguage();
            $newidartlang->loadByArticleAndLanguageId( $thisidart, 1);
            $thisidartlang = $newidartlang->get("idartlang");

            // as it seems, this routine might index old/empty/unused module containers too
            // might happen, that neither idart or idartlang can be found
            // in case that happens, do NOT try to add this articles to Solr, as it just produces log errors

            if ( intval($thisidartlang) != 0 && intval($thisidart) != 0 ) $artIncIdartlangs[] = $thisidartlang;
            //print "idartlang:". $newidartlang->get("idartlang") ."  xx  ";
            //$artIncIdartlangs[] = getidart->get("idart");

            //if ( intval($newidartlang->get("idartlang")) == 0 ) {
            //    print "<br>Fehler Zeile 302 :: idartlang " .$newidartlang->get("idartlang") ." :: idart " . $thisidart ;
            //    }

            }

            //print_r($thisraw);

            //    $image = $this->_getImageUrlByIdupl($idupl);
            //    if (false === $image) {
            //            //Util::log("skipped \$idupl: $idupl");
            //            continue;
            //    }
            //    $handleEntry->set('cms_raw', $image);

            }
            }

             */

            if (isset($articleContent['CMS_IMGEDITOR'])) {
                $contentSeparator = "";
                foreach ($articleContent['CMS_IMGEDITOR'] as $typeid => $idupl) {
                    if (0 == strlen($idupl)) {
                        continue;
                    }
                    $image = $this->_getImageUrlByIdupl($idupl);
                    if (false === $image) {
                        //Util::log("skipped \$idupl: $idupl");
                        continue;
                    }
                    $contentSeparator .= $image . "@@@";
                }

                $handleEntry->set('images', $contentSeparator);
            }

            // now add the WYSIWYG-version of the article too and change it slightly to prepare it for output

            define("COOKIE_FILE", "cookie.txt");

            // login page must be specific to client $articleIds['idclient']

            if ($this->_clients[$articleIds['idclient']]["loginidart"] != "") {
                $url2login = cUri::getInstance()->build(array(
                    'idart' => $this->_clients[$articleIds['idclient']]["loginidart"],
                    'lang' => $articleIds['idlang'],
                ), true); // true creates absolute path instead of relative

                $ch = curl_init();
            } else {
                $url2login = "";
            }

            //print "loginurl: " . $url2login;

            // die Frage ist, ob der ganze Block auskommentiert werden muss, falls es keinen Login gibt.
            curl_setopt($ch, CURLOPT_URL, $url2login);
            curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/32.0.1700.107 Chrome/32.0.1700.107 Safari/537.36');
            curl_setopt($ch, CURLOPT_POST, true);
            // für Cuckoo & LMS muss dies einem LMS-Benutzernamen entsprechen. Dieser sollte baugleich in Contenido vorhanden sein
            curl_setopt($ch, CURLOPT_POSTFIELDS, "username=" . $this->_feuUserName . "&password=" . $this->_feuUserPassword);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

            curl_setopt($ch, CURLOPT_COOKIESESSION, true);
            curl_setopt($ch, CURLOPT_COOKIEJAR, "1frontend");
            curl_setopt($ch, CURLOPT_COOKIEFILE, "1frontend");
            $answer = curl_exec($ch);

            $url2grab = cUri::getInstance()->build(array(
                'idart' => $articleIds['idart'],
                'lang' => $articleIds['idlang'],
            ), true); // true creates absolute path instead of relative
            // print $url2grab;
            curl_setopt($ch, CURLOPT_URL, $url2grab);
            curl_setopt($ch, CURLOPT_POST, false);
            curl_setopt($ch, CURLOPT_POSTFIELDS, "");
            $code = curl_exec($ch);
            $posStart = strpos($code, "<!--start:content-->");
            $posEnd = strpos($code, "<!--end:content-->");
            $difflen = $posEnd - $posStart;
            $html = substr($code, $posStart, $difflen);

            //Dieser Schritt ist nötig, um die Kursnavi zu eliminieren. Die Länderflagge bringt die PDF zum Absturz, warum auch immer, daher wird das einfach rausgeschnitten, basta.
            $posStart = strpos($html, "<!-- navigation_kurse_sub -->");
            $posEnd = strpos($html, "<!-- /navigation_kurse_sub -->");
            $difflen = $posEnd - $posStart;
            $interim = substr($html, $posStart, $difflen);
            $html = str_replace($interim, "", $html);
            // und nun zerstöre alle a-tags und mache p-tags daraus
            $html = str_replace("<a ", "<span ", $html);
            $html = str_replace("</a>", "</span>", $html);
            //$html = stripslashes($html );
            curl_close($ch);
            //print $html; // checked: content IS rendered, no errors here

            $handleEntry->set('renderedContent', $html);

            $strippedContent = strip_tags($html);
            $strippedContentDelete = array("\n", "\t", "\r");
            $strippedContent = str_replace($strippedContentDelete, " ", $strippedContent);
            $strippedContent = str_replace("&nbsp;", " ", $strippedContent);
            $strippedContent = str_replace("\"", "", $strippedContent);
            $strippedContent = trim(preg_replace('/\s+/', ' ', $strippedContent));
            $strippedContent = trim($strippedContent);
            $strippedContent = html_entity_decode($strippedContent);
            $handleEntry->set('strippedContent', $strippedContent);

            // add tagging categories as array to solr
            if ($this->_tagging) {
                $tagging = new pApiContentAllocation();
                //$handleEntry->set('taggingCategories', $tagging->loadAllocations($articleIds['idartlang']));
                //print "<pre>"; print_r($tagging->loadAllocations($articleIds['idartlang'])); print"</pre>";

                // add each tagging Category seperatly (content type fields are
                // defined as multiValued)

                $contentSeparator = "";
                foreach ($tagging->loadAllocations($articleIds['idartlang']) as $typeid => $contentEntry) {
                    $contentSeparator .= $contentEntry . ";";
                    //$handleEntry->set('taggingCategories', $contentEntry);
                    //if ( $articleIds['idart'] == 674) print $articleIds['idart']." ".$contentEntry."</br>";
                }

                $handleEntry->set('taggingCategories', $contentSeparator);

                $handleEntry->set('testIndex', "1");
            }

            //add this article to full array
            //array_push($toAdd[$articleIds['idlang']]['documents'], $solrInputDocument);

            //DELEEETE; then STOOOORE
            $handleEntry->store();

        }

        // add and commit documents and then optimze index
        /* old storing routine
    foreach ($toAdd as $idlang => $data) {
    try {
    $solrClient = $this->_getSolrClient($data['idclient'], $idlang);
    if (self::DBG) {
    error_log('# addArticles #');
    error_log('idclient: ' . $data['idclient']);
    error_log('idlang: ' . $idlang);
    error_log('config: ' . print_r($solrClient->getOptions(), 1));
    error_log('#documents: ' . count($data['documents']));
    } else {
    @$solrClient->addDocuments($data['documents']);
    @$solrClient->commit();
    @$solrClient->optimize();
    }
    } catch (Exception $e) {
    // log exception
    Solr::log($e);
    // rethrow as cException
    throw new cException('article could not be deleted from index', 0, $e);
    }
    }
     */

    }

    /*
     * @param $itemID int specifies the tour
     */
    public function delete($itemID)
    {
        return parent::delete($itemID);
    }
    /**
     * Gets path to upload.
     *
     * @param int $idupl
     */
    private function _getImageUrlByIdupl($idupl)
    {
        $upload = new cApiUpload($idupl);

        if (false === $upload->isLoaded()) {
            return false;
        }

        $idclient = $upload->get('idclient');
        $dirname = $upload->get('dirname');
        $filename = $upload->get('filename');

        $clientConfig = cRegistry::getClientConfig($idclient);
        $image = $clientConfig['upl']['htmlpath'] . $dirname . $filename;

        return $image;
    }

    public function handleStoringOfArticle(array $newData, array $oldData)
    {

        // get IDs of given article language
        if (cRegistry::getArticleLanguageId() == $newData['idartlang']) {
            // quite easy if given article is current article
            $idclient = cRegistry::getClientId();
            $idlang = cRegistry::getLanguageId();
            $idcat = cRegistry::getCategoryId();
            $idart = cRegistry::getArticleId();
            $idcatlang = cRegistry::getCategoryLanguageId();
            $idartlang = cRegistry::getArticleLanguageId();
        } else {
            // == for other articles these infos have to be read from DB
            // get idclient by idart
            $article = new cApiArticle($newData['idart']);
            if ($article->isLoaded()) {
                $idclient = $article->get('idclient');
            }
            // get idlang by idartlang
            $articleLanguage = new cApiArticleLanguage($newData['idartlang']);
            if ($articleLanguage->isLoaded()) {
                $idlang = $articleLanguage->get('idlang');
            }
            // get first idcat by idart
            $coll = new cApiCategoryArticleCollection();
            $idcat = array_shift($coll->getCategoryIdsByArticleId($newData['idart']));
            // get idcatlang by idcat & idlang
            $categoryLanguage = new cApiCategoryLanguage();
            $categoryLanguage->loadByCategoryIdAndLanguageId($idcat, $idlang);
            if ($categoryLanguage->isLoaded()) {
                $idcatlang = $articleLanguage->get('idlang');
            }
        }

        $thisArticle = array();
        array_push($thisArticle, array(
            'idclient' => $idclient,
            'idlang' => $idlang,
            'idcat' => $idcat,
            'idcatlang' => $idcatlang,
            'idart' => $idart,
            'idartlang' => $idartlang,
        ));
        //$this->updateSingleArticle($thisArticle);
        $this->_articleIds = $thisArticle;
        try {
            $this->deleteArticles();
        } catch (cException $e) {
            // ignore exception so that articles can be indexed nonetheless
        }
        $this->addArticles();

/*
self::handleStoringOfContentEntry(array(
'idclient' => $idclient,
'idlang' => $idlang,
'idcat' => $idcat,
'idcatlang' => $idcatlang,
'idart' => $idart,
'idartlang' => $idartlang
));
 */

    }
    public function handleStoringOfContentEntry(array $articleIds)
    {
        try {
            // build indexer instance
            $indexer = new self(array(
                $articleIds,
            ));
            // update given articles
            $indexer->updateArticles();
        } catch (cException $e) {
            /*
        $lvl = $e instanceof SolrWarning? cGuiNotification::LEVEL_WARNING : cGuiNotification::LEVEL_ERROR;
        $note = new cGuiNotification();
        $note->displayNotification($lvl, $e->getMessage());
         */
        }

        // destroy indexer to free mem
        unset($indexer);
    }
}

/**
 * Single Booking Item
 */
class Fullsearch extends Item
{

    /**
     * Constructor Function
     * @param string $table The table to use as information source
     */
    public function __construct($mId = false)
    {
        global $cfg;

        parent::__construct($cfg["tab"]["fullsearch"], "idfullsearch");
        if ($mId !== false) {
            $this->loadByPrimaryKey($mId);
        }
    }

    /**
     * Overridden setField method to md5 the password
     * Sets the value of a specific field
     *
     * @param string $field Specifies the field to set
     * @param string $value Specifies the value to set
     */
    public function setField($field, $value, $safe = true)
    {
        return parent::setField($field, $value, $safe);
    }

    public function store()
    {
        global $auth;

        return parent::store();
    }

}
