<?php
/**
 *
 * @package Plugin
 * @subpackage SearchSolr
 * @author Marcus Gnaß <marcus.gnass@4fb.de>
 * @copyright four for business AG
 * @link http://www.4fb.de
 */

// assert CONTENIDO framework
defined('CON_FRAMEWORK') || die('Illegal call: Missing framework initialization - request aborted.');

/**
 * This module handles a search request using a Solr searcher implementation and
 * displays the returned search results using a Smarty template.
 *
 * @author Marcus Gnaß <marcus.gnass@4fb.de>
 */

class FullsearchModule
{

    /**
     *
     * @var string
     */
    private $_searchTerm;

    /**
     *
     * @var string
     */
    protected $_dispSearchTerm = null;

    /**
     *
     * @var int
     */
    private $_page;

    /**
     *
     * @var int
     */
    private $_itemsPerPage;

    /**
     *
     * @var array
     */
    protected $_userInfo;

    /**
     *
     * @var array
     */
    protected $_label;

    /**
     *
     * @var string
     */
    private $_templateName;

    /**
     *
     * @var boolean
     */
    protected $_allowProtectedResults;

    /**
     *
     * @var FullsearchObject
     */
    private $_response = null;

    /**
     *
     * @var int
     */
    protected $_numberOfPages = null;

    /**
     *
     * @var string
     */
    protected $_url2login;

    /**
     *
     * @param array $options
     */
    public function __construct(array $options = null)
    {
        //print "start0...";
        if (null !== $options) {
            foreach ($options as $name => $value) {
                $name = '_' . $name;
                $this->$name = $value;
            }
        }
        //print "got response:<pre>";print_r($this);print "</pre>";
        // get global variables from registry (the nice way)
        $this->_cfg = cRegistry::getConfig();
        $this->_db = cRegistry::getDb();
        $this->_client = cRegistry::getClientId();
        $this->_lang = cRegistry::getLanguageId();
        $this->_idcat = cRegistry::getCategoryId();
        $this->_idart = cRegistry::getArticleId();
        $this->_sess = cRegistry::getSession();

        //print "start2...";
        $this->_response = $this->getSearchResults();
        //print "got response:<pre>";        print_r($this->_response);        print "</pre>";
        /*
    if ($this->_searchType != "handbuch" ) {
    $this->_response = $this->_getSearchResults();
    } else {
    $this->_response = $this->_getHandbuchResults();
    $this->_responseOrdered = $this->_orderResults();
    $this->_generatePDF();
    }
     */
    }

    /**
     *
     * @return SolrObject
     */
    private function _getSearchResults()
    {
        //$searcher = new SolrSearcherSimple();
        //$searcher->setSearchTerm(strtolower($this->_searchTerm));
        //$searcher->setSearchTerm($this->_searchTerm);
        //$searcher->setPage($this->_page);
        //$searcher->setItemsPerPage($this->_itemsPerPage);
        return $this->getSearchResults();
    }

    public function getSearchResults()
    {

        $idclient = cRegistry::getClientId();
        $idlang = cRegistry::getLanguageId();
        $auth = cRegistry::getAuth();
        $thisperm = array();

        if ($auth->auth["uid"] != "nobody") {
            $oFeUser = new cApiFrontendUser();
            $oFeUser->loadByPrimaryKey($auth->auth["uid"]);
            $thisFEU = $oFeUser->getGroupsForUser();
            //$oFeGroup = new cApiFrontendGroup();
            //print var_dump($thisFEU);
            foreach ($thisFEU as $thisgroup) {
                $thisperm[] = "accessGroups LIKE '%@" . $thisgroup . "@%'";
            }
            //$oFeGroup->loadByPrimaryKey($thisFEU[0]);
        }

        if (!$this->_allowProtectedResults) {

            if (!empty($thisperm)) {
                $searchStringAdditions = " AND (" . implode(" OR ", $thisperm) . " OR accessGroups LIKE '%@@@@@@%')";
            } else {
                $searchStringAdditions = " AND accessGroups LIKE '%@@@@@@%'";
            }

        } else {
            $searchStringAdditions = ""; // take them all, no matter what accessGroup
        }

        //print var_dump($searchStringAdditions);
        //$feupermission = new cApiFrontendPermissionCollection();
        //print $feupermission->checkPerm( 6, 'category', 'access', 1639 ) ;

        $searchTerm = $this->_searchTerm;
        //print " --- original searchTerm: " . $searchTerm;
        $searchTerm = trim($searchTerm);
        $searchTerm = str_replace(":", "", $searchTerm);
        $searchTerm = explode(' ', $searchTerm);
        $searchTerm = array_map('trim', $searchTerm);
        $searchTerm = array_filter($searchTerm);
        //print " --- searchTerm: <pre>"; print_r($searchTerm); print "</pre>";
        // $searchTerm = "cuckoo";
        //print var_dump($searchTerm);
        // there are no results if there is no search term
        if (!empty($searchTerm)) {
            $searchTermFinal = "strippedContent LIKE @@@" . implode("?????", $searchTerm) . "###";
            $searchTermFinal = str_replace("@@@", "'%", $searchTermFinal);
            $searchTermFinal = str_replace("###", "%'", $searchTermFinal);
            $searchTermFinal = str_replace("?????", "%' AND strippedContent LIKE '%", $searchTermFinal);

            //print " --- searchTermFinal: " . $searchTermFinal;
            //throw new cException('search cannot be performed for empty search term');

            $findResults = new FullsearchCollection();
            // Cuckoo Version:
            //$findResults->select($searchTermFinal . " AND id_client = " . $idclient . "  AND id_lang = " . $idlang . "  AND art_online = 1 AND art_searchable = 1 AND cat_visible = 1 AND accessGroups = 2 ORDER BY id_art DESC, id_cat ASC");
            $findResults->select($searchTermFinal . " AND id_client = " . $idclient . "  AND id_lang = " . $idlang . "  AND art_online = 1 AND art_searchable = 1 AND cat_visible = 1 " . $searchStringAdditions . " ORDER BY id_art DESC, id_cat ASC");

            $response = array();
            if ($findResults->count() > 0) {
                while ($thisresult = $findResults->next()) {
                    //$thisresult->CMS_HTMLHEAD = explode("@@@", $thisresult->CMS_HTMLHEAD);
                    $interimresult = $thisresult->toArray();
                    $interimresult["headlines"] = explode("@@@", $interimresult["CMS_HTMLHEAD"]);

                    // in case allowProtectedContent is set to true, some content must still be marked as "restricte"
                    // as not even every logged in user is allowed to see every content.
                    // Smarty must be able to determine which content ends as a "teaser" and not as a "linked content" in that instance.
                    // let's reuse $thisFEU (having all FEU groups of the logged in user) from above:
                    $interimresult["protectedContent"] = true;
                    foreach ($thisFEU as $thisgroup) {
                        if (strpos($interimresult["accessGroups"], $thisgroup)) {
                            $interimresult["protectedContent"] = false;
                        }

                    }

                    //print " --- interimresult: <pre>"; print_r($interimresult); print "</pre>";

                    $response[] = $interimresult;
                }
            }

            /*
            print " --- Results: " . $response . "<pre>";
            print_r($response);
            print "</pre>";
             */

            /* SolrQuery
            $query = new SolrQuery();
            // set the search query
            //$query->setQuery('strippedContent:*' . implode('* *', $searchTerm) . '*');
            //print 'renderedContent:*' . implode('* *', $searchTerm) . '* AND articleInclude:0 AND art_online:1';
            $query->setQuery('strippedContent:*' . implode('* *', $searchTerm) . '* AND articleInclude:0 AND art_online:1' . $searchStringAdditions);
            // specify the number of rows to skip
            $query->setStart(($this->_page - 1) * $this->_itemsPerPage);
            // specify the maximum number of rows to return in the result
            $query->setRows($this->_itemsPerPage);
            // specify fields to return
            // $query->addField('content');
            //$query->addField('renderedContent');
             */
            /* SolrClient
            $options = Solr::getClientOptions($idclient, $idlang);
            Solr::log(print_r($options, true));
            $solrClient = new SolrClient($options);
            // $solrClient->setServlet(SolrClient::SEARCH_SERVLET_TYPE,
            // $this->_servlet);

            $results = NULL;
            try {
            $solrQueryResponse = @$solrClient->query($query);
            $response = $solrQueryResponse->getResponse();
            $response = $response->response;
            } catch (SolrClientException $e) {
            Solr::log($e, $e->getFile(), $e->getLine());
            Solr::log($solrClient->getDebug());
            Solr::log($query->toString());
            }
             */

            return $response;
        } else {
            return false;
        }
    }

    private function _getHandbuchResults()
    {
        $searcher = new SolrSearcherSimple();
        $searcher->setSearchTerm($this->_searchTerm);
        //$searcher->setPage($this->_page);
        //$searcher->setItemsPerPage($this->_itemsPerPage);
        return $searcher->getHandbuchResults();
    }

    private function _orderResults()
    {

        global $treesort, $sortordertree, $brancheNames, $landNames, $themaNames;
        //print "<pre>"; print_r($treesort); print "</pre>";

        $rawResults = (array) $this->_response->docs;
        //print "<pre>"; print_r($rawResults); print "</pre>";

        $newtree = array();

        $searchcomparetagging = $this->_searchcomparetagging;
        //print "searchcomparetagging<pre>"; print_r($searchcomparetagging); print "</pre>";

        $brancheNames = "";
        $landNames = "";
        $themaNames = "";

        foreach ($rawResults as $key => $result) {
            // nicht vergessen: derzeit kommen ALLE values als Array gekapselt an statt direkt als Wert
            // print $key ." xx " . $result[0];

            $branche = array();
            $land = array();
            $thema = array();
            $thisTagCats = $result["taggingCategories"];
            //print "thisTagCats<pre>"; print_r($thisTagCats); print "</pre>";
            //print "searchcomparetagging<pre>"; print_r($searchcomparetagging); print "</pre>";

            // durchlaufe hier die taggings des konkreten Artikels und sortiere die Inhalte in ein neues Array in tree/branch-Version
            foreach ($searchcomparetagging as $key2 => $value) {

                if (in_array($value, $thisTagCats)) {
                    //print $value . "<br>";

                    if ($treesort[$value]["1mainkey"] == 0 && $treesort[$value]["1name"] != "") {$branche[] = $treesort[$value]["1sortorder"];
                        $brancheNames[$treesort[$value]["1sortorder"]] = $treesort[$value]["1name"];}
                    if ($treesort[$value]["2mainkey"] == 1 && $treesort[$value]["2name"] != "") {$land[] = $treesort[$value]["2sortorder"];
                        $landNames[$treesort[$value]["2sortorder"]] = $treesort[$value]["2name"];}
                    if ($treesort[$value]["3mainkey"] == 2 && $treesort[$value]["3name"] != "") {$thema[] = $treesort[$value]["3sortorder"];
                        $themaNames[$treesort[$value]["3sortorder"]] = $treesort[$value]["3name"];}
                }

            }

            //print "contentSorted<pre>"; print_r($branche); print "</pre>";

            foreach ($branche as $branchekey => $thisbranche) {
                foreach ($land as $landkey => $thisland) {
                    foreach ($thema as $themakey => $thisthema) {
                        $interimnewtree = array();
                        $treesortbranche = $sortordertree[0][$thisbranche];
                        //multidimensional_search($treesort, array('mainkey'=>0, 'sortorder'=>2));
                        //array_search( $thisbranche, array_column($treesort, 'sortorder') );
                        $treesortland = $sortordertree[1][$thisland];
                        $treesortthema = $sortordertree[2][$thisthema];

                        //print $treesortbranche ." xx ". $treesortland." xx ".$treesortthema." <br> ";
                        //print $thisbranche ." xx ". $thisland." xx ".$thisthema." <br> ";
                        //$content = str_replace ("<a ", "<span ", $rawResults[$resultid]["renderedContent"][0] );
                        //$content =  str_replace ("</a>", "</span>", $content);

                        $interimnewtree = array("resultid" => $key);
                        $interimnewtree = array_merge($interimnewtree, array("ArtSortID" => $rawResults[$key]["sortID"][0]));
                        $interimnewtree = array_merge($interimnewtree, array("content" => $rawResults[$key]["renderedContent"][0]));
                        $interimnewtree = array_merge($interimnewtree, $treesort[$treesortbranche]);
                        $interimnewtree = array_merge($interimnewtree, $treesort[$treesortland]);
                        $interimnewtree = array_merge($interimnewtree, $treesort[$treesortthema]);
                        if (!empty($interimnewtree)) {
                            $newtree[] = $interimnewtree;
                        }

                        //ksort ($newtree);

                    }

                }
            }

        }

        //print "<pre>"; print_r($rawResults); print "</pre>";
        //print "contentSorted<pre>"; print_r($newtree); print "</pre>";

        // jetzt wird das neue Array noch final umsortiert

        foreach ($newtree as $key => $array) {
            $sortorder1Array[$key] = $array["1sortorder"];
            $sortorder2Array[$key] = $array["2sortorder"];
            $sortorder3Array[$key] = $array["3sortorder"];
            $sortorder4Array[$key] = $array["ArtSortID"];
        }

        array_multisort($sortorder1Array, SORT_ASC, SORT_NUMERIC, $sortorder2Array, SORT_ASC, SORT_NUMERIC, $sortorder3Array, SORT_ASC, SORT_NUMERIC, $sortorder4Array, SORT_ASC, SORT_NUMERIC, $newtree);

        $newtree2 = array();
        foreach ($newtree as $key => $value) {
            $newtree2[$value["1sortorder"]][$value["2sortorder"]][$value["3sortorder"]][] = $value;
        }

        // sort all tagging categories according to their appearance in the form and tagging tree
        ksort($brancheNames);
        ksort($landNames);
        ksort($themaNames);
        //print "<pre>";print_r($newtree2); print "</pre>";
        return $newtree2;
    }

    /**
     *
     */
    public function render()
    {
        //print "--- start rendering --";

        $this->_dispSearchTerm = $this->_searchTerm;

        // calculate number of pages

        $numPages = count($this->_response) / $this->_itemsPerPage;

        if (is_float($numPages)) {
            $numPages = ceil($numPages);
        }
        $this->_numberOfPages = $numPages;

        $lower = ($this->_page - 1) * $this->_itemsPerPage;
        $upper = $lower + $this->_itemsPerPage - 1;
        $total = count($this->_response);
        $this->_msgRange = sprintf($this->_label['msgRange'], $lower, $upper, $total);

        if (0 === count($this->_response)) {
            $this->_msgResult = sprintf($this->_label['msgNoResultsFound'], $this->_dispSearchTerm, count($this->_response));
        } else {
            $this->_msgResult = sprintf($this->_label['msgResultsFound'], $this->_dispSearchTerm, count($this->_response));
        }

        /* nun noch die nötigen Elemente für page und die Anzahl der anzuzeigenden Elemente rausfiltern. */
        $responsePart = array_slice($this->_response, $lower, $this->_itemsPerPage, false);

        // Create an alternative Array for Pagination, imitating ALR version 04

        // Example: << < 1 2 3 ... 5 6 7 ... 9 10 11 > >>

        $lang = $this->_lang;
        $idcat = $this->_idcat;
        $idart = $this->_idart;
        $iPages = count($this->_getPageLinks());

        $sPaginationType04 = array();
        if ($this->_page != 1) {
            $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => 1, 'search_term' => $this->_searchTerm);
            $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), "«");
        } else {
            $sPaginationType04[] = array('', "«");
        }

        if (($this->_page - 1) >= 1) {
            $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => ($this->_page - 1), 'search_term' => $this->_searchTerm);
            $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), "‹");
        } else {
            $sPaginationType04[] = array('', "‹");
        }

        for ($i = 1; $i <= $iPages; $i++) {
            $aStartRange = array(1, 2, 3);
            $aEndRange = array($iPages, ($iPages - 1), ($iPages - 2));
            if (in_array($i, $aStartRange) || in_array($i, $aEndRange)) {
                $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => $i, 'search_term' => $this->_searchTerm);
                if ($this->_page != $i) {
                    $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), $i);
                } else {
                    $sPaginationType04[] = array('active', $i); // I'm on the current page, no link
                }
            } elseif ($this->_page == ($i + 1)) {
                // Front
                $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => $i, 'search_term' => $this->_searchTerm);
                $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), $i);
            } elseif ($this->_page == ($i - 1)) {
                // Back
                $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => $i, 'search_term' => $this->_searchTerm);
                $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), $i);
            } elseif ($this->_page == ($i - 2) ||
                $this->_page == ($i + 2) ||
                ($this->_page == ($i - 3) && (($i - 3) == 1)) ||
                ($this->_page == ($i + 3) && (($i + 3) == $iPages))) {
                $sPaginationType04[] = array('', ' ...');
            } elseif ($this->_page == ($i)) {
                $sPaginationType04[] = array('active', $i); // I'm on the current page, no link
            } else {
                $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => $i, 'search_term' => $this->_searchTerm);
                //$sPaginationType04[]   = array( cUri::getInstance()->buildRedirect($aParams), $i );
            }
        }

        if (($this->_page + 1) <= $iPages) {
            $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => ($this->_page + 1), 'search_term' => $this->_searchTerm);
            $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), "›");
        } else {
            $sPaginationType04[] = array('', "›");
        }

        if ($this->_page != $iPages) {
            $aParams = array('lang' => $lang, 'idcat' => $idcat, 'idart' => $idart, 'page' => $iPages, 'search_term' => $this->_searchTerm);
            $sPaginationType04[] = array(cUri::getInstance()->buildRedirect($aParams), "»");
        } else {
            $sPaginationType04[] = array('', "»");
        }
        array_push($sPagination, $sPaginationType04);

        $tpl = cSmartyFrontend::getInstance();
        $tpl->assign('userInfo', $this->_userInfo);
        $tpl->assign('label', $this->_label);
        $tpl->assign('href', cUri::getInstance()->build(array(
            'idart' => cRegistry::getArticleId(),
            'lang' => cRegistry::getLanguageId(),
        )));
        $tpl->assign('searchTerm', $this->_searchTerm);
        $tpl->assign('page', $this->_page);
        $tpl->assign('itemsPerPage', $this->_itemsPerPage);

        $tpl->assign('msgResult', $this->_msgResult);
        $tpl->assign('msgRange', $this->_msgRange);
        $tpl->assign('prev', $this->_getPreviousLink());
        $tpl->assign('next', $this->_getNextLink());
        $tpl->assign('pages', $this->_getPageLinks());
        $tpl->assign('tree', $this->_tree);
        $tpl->assign('url2login', $this->_url2login);
        $tpl->assign('allowProtectedResults', $this->_allowProtectedResults);
        $tpl->assign('pagination04', $sPaginationType04);

        //print "<pre>"; print_r($this->_getPageLinks());print "</pre>";

        $tpl->assign('numPages', $numPages);
        $tpl->assign('numFound', count($this->_response));
        //$tpl->assign('start', $this->_response->start);
        if (false === $this->_response) {
            $tpl->assign('results', array());
        } else {
            //$thisResult = json_decode(str_replace($this->_searchTerm, "sososososo",  json_encode($this->_response->docs)));
            $thisResult = (array) $this->_response;
            //var_dump($thisResult);
            //array_walk_recursive($thisResult , 'custom_replace', $this->_searchTerm );
            //print "yo";
            //$thisResult = str_replace($this->_searchTerm, "sososososo", $this->_response->docs);
            $tpl->assign('results', $responsePart);
        }

        $tpl->display($this->_templateName);
    }

    public function renderHandbuch()
    {

        global $treesort;

        //print "<pre>"; print_r($this->_response->docs); print "</pre>";
        $this->_dispSearchTerm = $this->_searchTerm;

        if (false === $this->_response->docs) {
            //$tpl->assign('results', array());
        } else {
            //$thisResult = json_decode(str_replace($this->_searchTerm, "sososososo",  json_encode($this->_response->docs)));
            $thisResult = (array) $this->_response->docs;
            //var_dump($thisResult);
            //array_walk_recursive($thisResult , 'custom_replace', $this->_searchTerm );
            //print "yo";
            //$thisResult = str_replace($this->_searchTerm, "sososososo", $this->_response->docs);
            //$tpl->assign('results', $thisResult);
            //$tpl->assign('resultorder', $this->_responseOrdered);
            //$tpl->assign('treesort', $treesort);
        }

        //$tpl->display($this->_templateName);
        //$this->_pdfOutput = $tpl->fetch($this->_templateName);
    }

    /**
     *
     * @return string
     */
    protected function _getPreviousLink()
    {

        // skip if there are no previous pages
        if (1 >= $this->_page) {
            return '';
        }

        // build link to previous result page
        $url = $this->_getPageLink($this->_dispSearchTerm, $this->_page - 1);

        return $url;
    }

    /**
     *
     * @return string
     */
    protected function _getNextLink()
    {

        // skip if there are no next pages
        if ($this->_page >= $this->_numberOfPages) {
            return '';
        }

        // build link to next result page
        $url = $this->_getPageLink($this->_dispSearchTerm, $this->_page + 1);

        return $url;
    }

    /**
     * Build links to other result pages.
     *
     * @return string
     */
    protected function _getPageLinks()
    {
        $pageLinks = array();
        for ($i = 1; $i <= $this->_numberOfPages; $i++) {
            $pageLinks[$i] = $this->_getPageLink($this->_dispSearchTerm, $i);
        }

        return $pageLinks;
    }

    /**
     * This method builds URLs for each result link and the pagination links.
     *
     *
     * @param string $searchTerm
     * @param int $page
     * @return mixed
     */
    protected function _getPageLink($searchTerm = null, $page = null)
    {

        // define standard params
        $params = array(
            'lang' => $this->_lang,
            'idcat' => $this->_idcat,
            'idart' => $this->_idart,
        );
        // add optional params if given
        if (null !== $searchTerm) {
            $params['search_term'] = conHtmlEntityDecode($searchTerm);
        }
        if (null !== $page) {
            $params['page'] = $page . $this->_artSpecs;
        }
        // store unaltered params for later use
        $defaultParams = $params;

        // define special params when 'front_content' or 'MR' url builders are
        // *NOT* used in this case the standard params are wrapped as 'search'
        // and lang, idcat & level are aded cause they are needed to build the
        // category path
        $url_builder = array(
            'front_content',
            'MR',
        );
        if (false === in_array($this->_cfg['url_builder']['name'], $url_builder)) {
            $params = array(
                'search' => $params,
                'lang' => $this->_lang,
                'idcat' => $this->_idcat,
                'level' => 1,
            );
        }

        try {
            $url = cUri::getInstance()->build($params);
        } catch (cInvalidArgumentException $e) {
            $url = $this->_sess->url(implode('?', array(
                'front_content.php',
                implode('&', $defaultParams),
            )));
        }

        return $url;
    }

    protected function _generatePDF()
    {

        global $bcvar, $lmsSite, $brancheNames, $landNames, $themaNames;

        cInclude('frontend', 'includes/mpdf/mpdf.php');
        //$rawResults = (array)$this->_response->docs;

        $html = "";

        //$mpdf = new mPDF();
        $mpdf = new mPDF('utf-8', 'A4', '', '', '15', '15', '40', '18');
        $mpdf->img_dpi = 300;
        //$mpdf->allow_output_buffering = true;
        //$mpdf->debug = true;
        $mpdf->SetHTMLHeader('<table id="mpdf_header"><tr><td class="tdlogo1"><img class="logo" src="' . $lmsSite . '/images/logo_logbuch_pdf2.jpg"></td><td class="tdlogo2" style="text-align:right;"><img class="logo2" src="' . $lmsSite . '/images/logo_mrh_pdf.jpg"></td></tr></table>');
        $mpdf->SetHTMLFooter('<div id="mpdf_footer" class="mpdf_handbuch"><div class="mpdf_www">' . $lmsSite . '</div><div class="mpdf_date">Datum: ' . $bcvar["today"] . '</div></div>', 'O');

        //$mpdf->allow_charset_conversion=true;
        //$mpdf->charset_in='UTF-8';
        $mpdf->useSubstitutions = true; // optional - just as an example
        //$mpdf->SetHeader($pageprint . "\n\n" . 'Page {PAGENO}');  // optional - just as an example

        $mpdf->CSSselectMedia = 'print'; // assuming you used this in the document header
        //$mpdf->setBasePath($url2print);
        $mpdf->setBasePath($lmsSite);
        $stylesheet = file_get_contents('css/mpdf.css');

        $mpdf->WriteHTML($stylesheet, 1);

        // Deckblatt
        //$mpdf->AddPage();
        $html = '<div class="pdfhandbuch">';
        $html .= '<div style="font-size: 40px; padding-top: 50mm; padding-bottom: 20mm;">Ihr persönliches Handbuch<br>zum internationalen Gast</div>';
        $html .= '<div style="font-size: 24px; padding-bottom: 20mm;">' . $this->_userInfo["firstName"] . ' ' . $this->_userInfo["lastName"] . '</div>';
        $html .= '<div style="font-size: 16px; padding-bottom: 10mm; font-weight: bold;">Für die Themen:</div>';
        $html .= '<div style="font-size: 16px;">' . implode(" - ", $brancheNames) . '</div>';
        $html .= '<div style="font-size: 16px;">' . implode(" - ", $landNames) . '</div>';
        $html .= '<div style="font-size: 16px;">' . implode(" - ", $themaNames) . '</div>';
        $html .= '</div>';
        $mpdf->WriteHTML($html);

        // start output bit by bit

        foreach ($this->_responseOrdered as $keybranche => $branche) {
            foreach ($branche as $keyland => $land) {
                foreach ($land as $keythema => $thema) {

                    $mpdf->AddPage();
                    $html = "";

                    foreach ($thema as $keyinhalt => $inhalt) {

                        $mpdf->SetHTMLFooter('<div id="mpdf_footer" class="mpdf_handbuch"><div class="mpdf_www">' . $inhalt["1name"] . ' | ' . $inhalt["2name"] . ' | ' . $inhalt["3name"] . '</div><div class="mpdf_date">Datum: ' . $bcvar["today"] . '</div></div>', 'O');
                        $intro = '<div id="mpdf_header2" class="mpdf_handbuch">Branche: ' . $inhalt["1name"] . ' | Land: ' . $inhalt["2name"] . ' | Thema: ' . $inhalt["3name"] . '</div>';
                        $html .= personalizeText($inhalt["content"]);
                    }

                    $mpdf->WriteHTML($intro . $html);

                }
            }

        }

        $mpdf->Output('LogbuchInternational_Handbuch_' . $bcvar["todaypdf"] . '.pdf', 'D');

        //return $url;
    }

}
