<?php

/**
 *
 * @package Plugin
 * @subpackage T2T Full Search
 * @author Viktor Lehmannn & <marcus.gnass@4fb.de>
 * @copyright TONE2TONE & four for business AG
 */

// assert CONTENIDO framework
defined('CON_FRAMEWORK') || die('Illegal call: Missing framework initialization - request aborted.');

/**
 * Helper class for this plugin.
 *
 * @author Viktor Lehmann & Marcus Gnaß <marcus.gnass@4fb.de>
 */
 
$cfg["tab"]["fullsearch"] = $cfg['sql']['sqlprefix']."_fullsearch";
 
class T2TSearch {

    /**
     * name of this plugin
     *
     * @var string
     */
    private static $_name = 't2t_fullsearch';

    /**
     *
     * @param mixed $whatever
     */
    public static function log($whatever, $file = NULL, $line = NULL) {
        $msg = '';
        if ($whatever instanceof Exception) {
            $msg .= '========================' . PHP_EOL;
            $msg .= '=== LOGGED EXCEPTION ===' . PHP_EOL;
            $msg .= 'REFERER: ' . $_SERVER['HTTP_REFERER'] . PHP_EOL;
            $msg .= 'URI: ' . $_SERVER['REQUEST_URI'] . PHP_EOL;
            $msg .= 'MSG: ' . $whatever->getMessage() . PHP_EOL;
            $msg .= 'TRACE: ' . $whatever->getTraceAsString() . PHP_EOL;
            $msg .= '========================';
        } else {
            $msg = $whatever;
        }

        static $start = 0;
        if (0 == $start) {
            $start = microtime(true);
        }
        $delta = microtime(true) - $start;

        // create name of logfile
        $cfg = cRegistry::getConfig();
        $filename = $cfg['path']['contenido_logs'] . 'errorlog_fullsearch.txt';

        // extend message with optional prefix
        $prefix = number_format($delta * 1000, 0) . 'ms: ';
        if (NULL !== $file) {
            $prefix .= $file;
            if (NULL !== $line) {
                $prefix .= ':' . $line;
            }
            $prefix .= ' ';
        }

        // log message
        $log = new cLog(cLogWriter::factory('file', array(
            'destination' => $filename
        )));
        $log->info($prefix . $whatever);
    }

    /**
     */
    public static function getName() {
        return self::$_name;
    }

    /**
     * Return path to this plugins folder.
     *
     * @return string
     */
    public static function getPath() {
        $cfg = cRegistry::getConfig();

        $path = cRegistry::getBackendPath() . $cfg['path']['plugins'];
        $path .= self::$_name . '/';

        return $path;
    }

    /**
     *
     * @param string $key
     * @return string
     */
    public static function i18n($key) {
        $trans = i18n($key, self::$_name);
        return $trans;
    }

    /**
     *
     * @param Exception $e
     */
    public static function logException(Exception $e) {
        $cfg = cRegistry::getConfig();

        $log = new cLog(cLogWriter::factory('file', array(
            'destination' => $cfg['path']['contenido_logs'] . 'errorlog_fullsearch.txt'
        )), cLog::ERR);

        $log->err($e->getMessage());
        $log->err($e->getTraceAsString());
    }

    /**
     * TODO build method to display erro & info box and just call it from here
     *
     * @param Exception $e
     * @param bool $showTrace if trace should be displayed too
     */
    public static function displayException(Exception $e, $showTrace = false) {
        header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);

        if (true) {
            // error box
            $class = "ui-state-error";
            $icon = "ui-icon-alert";
        } else {
            // info box
            $class = "ui-state-highlight";
            $icon = "ui-icon-info";
        }

        echo '<div class="ui-widget">';
        echo '<div class="' . $class . ' ui-corner-all">';
        echo '<p>';
        echo '<span class="ui-icon ' . $icon . '"></span>';
        // echo '<strong>Exception</strong>';
        echo $e->getMessage();
        if (true === $showTrace) {
            echo '<pre style="overflow: auto">';
            echo htmlentities($e->getTraceAsString(), ENT_COMPAT | ENT_HTML401, 'UTF-8');
            echo '</pre>';
        }
        echo '</p>';
        echo '</div>';
        echo '</div>';
    }

    /**
     * Creates a notification widget in order to display an exception message in
     * backend.
     *
     * @param Exception $e
     * @return string
     */
    public static function notifyException(Exception $e) {
        $cGuiNotification = new cGuiNotification();
        $level = cGuiNotification::LEVEL_ERROR;
        $message = $e->getMessage();

        return $cGuiNotification->returnNotification($level, $message);
    }
}

// define template names
//$cfg['templates']['fullsearch_right_bottom'] = $cfg['plugins'][T2TSSearch::getName()] . 'templates/template.right_bottom.tpl';

// include necessary sources, setup autoloader for plugin
$pluginClassPath = 'contenido/plugins/' . T2TSearch::getName() . '/';
cAutoload::addClassmapConfig(array(
    'FullsearchCollection' => $pluginClassPath . 'classes/class.fullsearch.php',
    'FullsearchModule' => $pluginClassPath . 'classes/class.fullsearch_results.php'
));
unset($pluginClassPath);
// == add chain functions
// reindex article after article properties are updated
$_cecRegistry = cApiCecRegistry::getInstance();
$_cecRegistry->addChainFunction('Contenido.Action.con_saveart.AfterCall', 'FullsearchCollection::handleStoringOfContentEntry');
// reindex article after any content entry is updated
$_cecRegistry->addChainFunction('Contenido.Content.AfterStore', 'FullsearchCollection::handleStoringOfContentEntry');

